#include <avr/io.h>

#include "ais326.h"
#include "ais326_config.h"

#define AIS326_CLOCK_HIGH	do {AIS326_SCK_PORT |= (1 << AIS326_SCK_BIT); }  while (0)
#define AIS326_CLOCK_LOW	do {AIS326_SCK_PORT &= ~(1 << AIS326_SCK_BIT);} while (0)

#define AIS326_SELECT		do {AIS326_CS_PORT &= ~(1 << AIS326_CS_BIT);} while (0)
#define AIS326_DESELECT		do {AIS326_CS_PORT |= (1 << AIS326_CS_BIT);} while (0)

#define AIS326_SEND_ONE		do {AIS326_MOSI_PORT |= (1 << AIS326_MOSI_BIT);} while (0)
#define AIS326_SEND_ZERO	do {AIS326_MOSI_PORT &= ~(1 << AIS326_MOSI_BIT);} while (0)

#define AIS326_SAMPLE		(AIS326_MISO_PIN & (1 << AIS326_MISO_BIT))

void ais326_init(void)
{
	AIS326_CS_PORT  |= (1 << AIS326_CS_BIT);
	AIS326_CS_DDR  |= (1 << AIS326_CS_BIT);
#if AIS326_SOFTWARE_SPI
	AIS326_SCK_PORT  |= (1 << AIS326_SCK_BIT);
	AIS326_MOSI_PORT  |= (1 << AIS326_MOSI_BIT);

	AIS326_SCK_DDR  |= (1 << AIS326_SCK_BIT);
	AIS326_MOSI_DDR  |= (1 << AIS326_MOSI_BIT);
	AIS326_MISO_DDR  &= ~(1 << AIS326_MISO_BIT);
#else
	AIS326_SS_PORT  |= (1 << AIS326_CS_BIT);
	AIS326_SS_DDR  |= (1 << AIS326_CS_BIT);
	SPCR = _BV(SPE) | _BV(MSTR) /*| _BV(SPR1)*/;
#endif

}


static uint8_t ais326_rx_tx(uint8_t data)
{
	uint8_t ret = 0;
#if AIS326_SOFTWARE_SPI
	uint8_t count = 8;
	while (count--) {
		ret <<= 1;
		AIS326_CLOCK_LOW;
		if (data & 0x80) AIS326_SEND_ONE;
		else AIS326_SEND_ZERO;
		if (AIS326_SAMPLE) ret |= 0x01;
		AIS326_CLOCK_HIGH;
		data <<= 1;
	}
	AIS326_SEND_ONE;
#else
	SPDR = data;
	while (!(SPSR & _BV(SPIF)));
	ret = SPDR;
#endif
	return ret;
}

uint8_t ais326_read(uint8_t reg)
{
	reg = (reg & 0x3F) | AIS326_READ | AIS326_SINGLE;
	AIS326_SELECT;
	ais326_rx_tx(reg);
	reg = ais326_rx_tx(0xFF);
	AIS326_DESELECT;
	return reg;
}

void ais326_write(uint8_t reg, uint8_t data)
{
	reg = (reg & 0x3F) | AIS326_WRITE | AIS326_SINGLE;
	AIS326_SELECT;
	ais326_rx_tx(reg);
	ais326_rx_tx(data);
	AIS326_DESELECT;
}

void ais326_read_multi(uint8_t reg, uint8_t *buf, uint8_t num)
{
	reg = (reg & 0x3F) | AIS326_READ | AIS326_MULTI;
	AIS326_SELECT;
	ais326_rx_tx(reg);
	for (;num; num--, buf++) {
		*buf = ais326_rx_tx(0xFF);
	}
	AIS326_DESELECT;
}


void ais326_write_multi(uint8_t reg, uint8_t *buf, uint8_t num)
{
	reg = (reg & 0x3F) | AIS326_WRITE | AIS326_MULTI;
	AIS326_SELECT;
	ais326_rx_tx(reg);
	for (;num; num--, buf++) {
		ais326_rx_tx(*buf);
	}
	AIS326_DESELECT;
}

void ais326_get_xyz(int16_t *xyz)
{
	ais326_read_multi(OUTX_L, (uint8_t*)xyz, 6);
}



